#include <GL/glut.h>

// some types for later use
typedef unsigned char byte;
typedef byte rgb[3];			// these are 3 byte pixels in RGB order

// a few colors to use
static rgb red = { 255, 0, 0 };
static rgb blue = { 0, 0, 255 };

// an object to represent the framebuffer where I draw
struct FrameBuffer {
	int width, height;
	rgb *pix;

	FrameBuffer(int w, int h) { width=w; height=h; pix = new rgb[width*height]; }
	void Clear(const rgb p);

} *fb; // global framebuffer object for use in callbacks

void FrameBuffer::Clear(const rgb p)
{
	register byte r = p[0];
	register byte g = p[1];
	register byte b = p[2];
	register byte* p0 = pix[0];
	register byte* p1 = p0 + sizeof(rgb)*width*height;
	while(p0 < p1) {
		p0[0] = r;
		p0[1] = g;
		p0[2] = b;
		p0 += 3;
	}
}

void myDisplay(void)
{
	// copy the framebuffer to the screen
	glDrawPixels(fb->width, fb->height, GL_RGB, GL_UNSIGNED_BYTE,  fb->pix);
	glFlush();
}

void myDraw(int flag)
{
	// I do my drawing into the framebuffer object here
	// this is just a hack example
	fb->Clear(flag ? red : blue);

	// then transfer to the screen
	// I can do this as often as I like, even after every pixel for debugging purposes.
	myDisplay();
}

void myInit()
{
	// initialize GL stuff for speed

#if 1
	// this makes quite a difference on the PC with software GL
	glDisable(GL_DITHER);
#endif

	// SGI claims some of these might produce speedups for some architectures
#if 0
    glDisable(GL_ALPHA_TEST);
    glDisable(GL_BLEND);
    glDisable(GL_DEPTH_TEST);
    glDisable(GL_FOG);
    glDisable(GL_LIGHTING);
    glDisable(GL_LOGIC_OP);
    glDisable(GL_STENCIL_TEST);
    glDisable(GL_TEXTURE_1D);
    glDisable(GL_TEXTURE_2D);
    glPixelTransferi(GL_MAP_COLOR, GL_FALSE);
    glPixelTransferi(GL_RED_SCALE, 1);
    glPixelTransferi(GL_RED_BIAS, 0);
    glPixelTransferi(GL_GREEN_SCALE, 1);
    glPixelTransferi(GL_GREEN_BIAS, 0);
    glPixelTransferi(GL_BLUE_SCALE, 1);
    glPixelTransferi(GL_BLUE_BIAS, 0);
    glPixelTransferi(GL_ALPHA_SCALE, 1);
    glPixelTransferi(GL_ALPHA_BIAS, 0);
#endif
}

void KeyInputHandler(unsigned char Key, int x, int y)
{
  switch(Key)
  {
    case 27: exit(0);
	default: 
		for(int i=0; i<500; i++) {
			myDraw(i%2==0); 
		}
		break;
  };
}


void main(int argc, char* argv[])
{
	int width=256, height=256;

	fb = new FrameBuffer(width, height);

    glutInitDisplayMode(GLUT_RGB);
	glutInitWindowPosition(50, 50);
    glutInitWindowSize(width, height);

    glutCreateWindow("OpenGL skeleton");

    glutKeyboardFunc(KeyInputHandler);

    glutDisplayFunc(myDisplay);

	myInit();

    glutMainLoop();
}


